/*
 * ParseAll.java
 * Copyright (C) 2004-2005, Bjorn Bringert (bringert@cs.chalmers.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package se.chalmers.cs.gf.parse;

import se.chalmers.cs.gf.GFException;
import se.chalmers.cs.gf.abssyn.Tree;
import se.chalmers.cs.gf.util.Pair;
import static se.chalmers.cs.gf.util.Pair.pair;
import static se.chalmers.cs.gf.parse.ParseLogger.*;

import java.util.*;

/**
 *  Try parsing with a collection of parsers.
 */
public class ParseAll implements Parser {

        private Parsers parsers;

        /**
         *  @param parsers The parsers that should be used to 
         *  parse with.
         */
        public ParseAll(Parsers parsers) {
                this.parsers = parsers;
        }

        public Collection<Tree> parse(String str) {
                Collection<Pair<String,Tree>> ts = parseWithAll(parsers, str);
                List<Tree> rs = new LinkedList<Tree>();
                for (Pair<String,Tree> p : ts) 
                        rs.add(p.snd);
                return rs;
        }

        /**
         *  Ignores the category argument.
         */
        public Collection<Tree> parse(String str, String cat) {
                return parse(str);
        }

        public String getName() {
                return "*all*";
        }

        public boolean tryThisOne() {
                return false;
        }

        /**
         *  Try parsing the input with all the available parsers.
         *
         *  @return A list of pairs (language, parse tree)
         */
        public static Collection<Pair<String,Tree>> parseWithAll(Parsers parsers, String str) {
                log.fine("Trying to parse '" + str + "' with all languages");

                Collection<Pair<String,Tree>> rs 
                        = new LinkedList<Pair<String,Tree>>();
                for (String l: parsers.listParsers()) {
                        Parser p = parsers.getParser(l);
                        if (p.tryThisOne()) {
                                try {
                                        log.finer("Trying " + l);
                                        for (Tree t : p.parse(str)) {
                                                log.fine(l + ": " + t);
                                                rs.add(pair(l, t));
                                        }
                                } catch (GFException ex) {
                                        log.error(ex);
                                }
                        }
                }
                return rs;
        }

}
