/*
 * SimpleLexer.java
 * Copyright (C) 2004-2005, Bjorn Bringert (bringert@cs.chalmers.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package se.chalmers.cs.gf.parse;

import java.io.*;
import java.util.*;
import java.util.regex.*;

import static se.chalmers.cs.gf.parse.ParseLogger.*;

/**
 *  A simple lexer.
 */
public class SimpleLexer implements Lexer {

        private Pattern tokenPatt;

        public SimpleLexer() {
                // simple words are any non-empty sequence of 
                // lower case letters, upper case letters, 
                // title case letters, other letters,
                // modifier letters, modifier symbols, 
                // decimal digits, letter digits,
                // other digits and connector punctuation.
                String word = "[\\p{Ll}\\p{Lu}\\p{Lt}\\p{Lo}\\p{Lm}\\p{Sk}\\p{Nd}\\p{Nl}\\p{No}\\p{Pc}]+";
                // quoted words are strings of any character non double quote
                // enclosed in double quotes
                // FIXME: allow escaped double quotes and backslashes
                String quoted = "\"(\\\\\"|[^\"])*\"";
                // punctuation is a single punctuation character
                // punctuation characters are:
                // - opening punctuation
                // - clsoing punctuation 
                // - dashes
                // - other punctuation
                // - currency symbols
                // - mathematic symbols
                String punct = "[\\p{Ps}\\p{Pe}\\p{Pd}\\p{Po}\\p{Sc}\\p{Sm}]";
                tokenPatt = Pattern.compile("(" + word 
                                            + "|" + quoted 
                                            + "|" + punct 
                                            + ")");
        }

        // FIXME: discards input that it doesn't understand
        public List<Token> lex(CharSequence input) {
                List<Token> ts = new ArrayList<Token>();

                Matcher m = tokenPatt.matcher(input);
                while (m.find()) {
                        String s = m.group();
                        ts.add(new Token(s));
                }

                log.finest("Lexed \"" + input + "\" to " + ts);

                return ts;
        }

        public static void main(String[] args) throws java.io.IOException {
                Lexer l = new SimpleLexer();
                BufferedReader in 
                        = new BufferedReader(new InputStreamReader(System.in));
                String line;
                while ((line = in.readLine()) != null)
                        System.out.println(l.lex(line));
        }

}
