/*
 * Chart.java
 * Copyright (C) 2004-2005, Bjorn Bringert (bringert@cs.chalmers.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package se.chalmers.cs.gf.parse.chart;

import java.util.*;

/**
 *  A parse chart.
 */
public class Chart {

        private Set<Edge>[][] chart;

        public Chart(int n) {
                // causes an unchecked cast warning. The cast is neccessary
                // since we cannot create arrays of parametrized types.
                // See section 7.3 of
                // http://java.sun.com/j2se/1.5/pdf/generics-tutorial.pdf
                // FIXME: array is almost twice as big as a it needs to be.
                //        could fix this with a non-square array
                chart = (Set<Edge>[][])new Set<?>[n+1][n+1];
                for (int i = 0; i <= n; i++)
                        for (int j = i; j <= n; j++)
                                chart[i][j] = new HashSet<Edge>();
        }

        /**
         *  @return true if the edge was not already in the set, i.e. if it was added.
         */
        public boolean add(Edge e, int i, int j) {
                if (i > j)
                        throw new IllegalArgumentException("i > j");

                boolean addedNew = chart[i][j].add(e);
//                if (addedNew)
//                        System.err.println("("+i+","+j+"): Added: " + e);
                return addedNew;
        }

        public Set<Edge> get(int i, int j) {
                if (i > j)
                        throw new IllegalArgumentException("i > j");

                return chart[i][j];
        }

        /**
         *  Get the input length that this chart is for.
         */
        public int getSize() {
                return chart.length-1;
        }

        /**
         *  Remove all active edges.
         */
        public void purgeActive() {
                int n = chart.length-1;
                for (int i = 0; i <= n; i++) {
                        for (int j = i; j <= n; j++) {
                                Iterator<Edge> it = chart[i][j].iterator();
                                while (it.hasNext())
                                        if (it.next().isActive())
                                                it.remove();
                        }
                }
        }


        public String toString() {
                StringBuilder sb = new StringBuilder();
                int n = getSize();
                for (int i = 0; i <= n; i++)
                        for (int j = i; j <= n; j++)
                                for (Edge e : get(i,j)) {
                                        sb.append("(").append(i).append(",");
                                        sb.append(j).append("): ");
                                        sb.append(e).append("\n");
                                }
                return sb.toString();
        }

}
