/*
 * EmptyRule.java
 * Copyright (C) 2004-2005, Bjorn Bringert (bringert@cs.chalmers.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package se.chalmers.cs.gf.parse.chart;

import java.util.*;

/**
 *  An empty CF grammar production.
 */
public class EmptyRule extends Rule {

        private String fun;

        private int argCount;

        /**
         * @param argCount The number of arguments needed by the function
         *  that this rule represents.
         */ 
        public EmptyRule(Category cat, String fun, int argCount) {
                super(cat);
                this.fun = fun;
                this.argCount = argCount;
        }

        public String getFun() {
                return fun;
        }

        public int getSize() {
                return 0;
        }

        public boolean isTerminal() {
                return true;
        }

        public int getArgCount() {
                return argCount;
        }

        public boolean equals(Object o) {
                return o instanceof EmptyRule && equals((EmptyRule)o);
        }
        
        public boolean equals(EmptyRule r) {
                // we don't need to compare argCount, since it is
                // determined by the function
                return getCategory().equals(r.getCategory())
                        && fun.equals(r.getFun());
        }

        public int hashCode() {
                return getCategory().hashCode() + fun.hashCode();
        }

        public String toString() {
                StringBuilder sb = new StringBuilder();
                sb.append(getFun());
                sb.append("[");
                for (int i = 0; i < argCount; i++) {
                        sb.append("[]");
                        if (i < argCount-1)
                                sb.append(",");
                }
                sb.append("]");
                sb.append(". ");
                sb.append(getCategory());
                sb.append(" -> ");

                return sb.toString();
        }

}
